/*
* Copyright (C) 2001-2008, CompHEP Collaboration
* Copyright (C) 1997, Alexander Pukhov 
* Made by      S.Shichanin
* at           03 April 1991
* Last update  26 June  1991
* ------------------------------------------------------
* $Id$
*
* $Log$
*/
#include <math.h>

#include "service2/include/chep_limits.h"
#include "service2/include/tptcmac.h"
#include "service2/include/4_vector.h"
#include "chep_crt/include/chep_crt.h"

#include "plot/include/plot.h"

#include "err_code.h"
#include "num_serv.h"
#include "const.h"
#include "out_ext.h"
#include "param.h"
#include "evnt_menu.h"
#include "width_12.h"

typedef struct canal_12
  {
    char prtclnames[2][6];
    double width;
  }
canal_12;

static canal_12 *allcanal;

static void 
sortcanals (void)
{
  int i;
  canal_12 buff;

  i = 0;
  while (i < nprc_ - 1)
    {
      if (allcanal[i].width < allcanal[i + 1].width)
	{
	  buff = allcanal[i];
	  allcanal[i] = allcanal[i + 1];
	  allcanal[i + 1] = buff;
	  if (i != 0)
	    i--;
	}
      else
	i++;
    }
}


static void 
decay12information (double locw12)
{
  byte xcount, ycount;
  int i;

  clrbox (1, 16, 80, 24);
  goto_xy (1, 16);
  scrcolor (Red, BGmain);
  print (" Total width : ");
  scrcolor (FGmain, BGmain);
  if (err_code == 0)
    print ("%13E GeV", locw12);
  else
    print (" incorrect       ");
  if (locw12 > 1.E-20 && err_code == 0)
    {
      sortcanals ();
      goto_xy (1, 17);
      scrcolor (Red, BGmain);
      print (" Modes and fractions :");
      xcount = 31;
      ycount = 18;
      for (i = 0; i < nprc_; i++)
	{
	  if (ycount < maxRow ())
	    {
	      goto_xy (xcount, ycount - 1);
	      scrcolor (Blue, BGmain);
	      print ("%s %s -  ", allcanal[i].prtclnames[0],
		     allcanal[i].prtclnames[1]);

	      scrcolor (FGmain, BGmain);
	      print ("  %5.2lG%%", 100 * allcanal[i].width / locw12);
	      xcount += 30;
	      if (xcount > maxCol ())
		{
		  xcount = 1;
		  ++(ycount);
		}
	    }
	}
    }
  scrcolor (FGmain, BGmain);
}


static double 
calcwidth12 (void)
{
  int i, nsub;
  double pmass[3];

  err_code = 0;
  double w12 = 0.0;

  for (nsub = 0; nsub < nprc_; ++nsub)
    {
      for (i = 0; i < 3; i++) {
	pinf_ (nsub + 1, i + 1, NULL, pmass + i);
      }

      if (pmass[0] <= pmass[1] + pmass[2]) {
	allcanal[nsub].width = 0.0;
      } else {
	double mm1 = pmass[0] * pmass[0];
	double mm2 = pmass[1] * pmass[1];
	double mm3 = pmass[2] * pmass[2];
	double pRestOut = sqrt ((mm1 - (mm2 + mm3 + 2. * pmass[1] * pmass[2])) *
	        	 (mm1 - (mm2 + mm3 - 2. * pmass[1] * pmass[2]))
	  ) / (2 * pmass[0]);
	double totcof = pRestOut / (8. * M_PI * mm1);

	for (i = 1; i < 12; i++)
	  pvect[i] = 0;
	pvect[0] = pmass[0];
	pvect[7] = pRestOut;
	pvect[4] = sqrt (pvect[7] * pvect[7] + mm2);
	pvect[11] = -pvect[7];
	pvect[8] = sqrt (pvect[7] * pvect[7] + mm3);

	allcanal[nsub].width = totcof * sqme_ (nsub + 1, pvect, &err_code);
      }
      w12 += allcanal[nsub].width;
      pinf_ (nsub + 1, 2, allcanal[nsub].prtclnames[0], NULL);
      pinf_ (nsub + 1, 3, allcanal[nsub].prtclnames[1], NULL);
    }

  if (err_code != 0) {
    errormessage ();
    return 0.;
  }

  return w12;
}


static double 
totwidth (void)
{
  return calcwidth12 ();
}


void 
decay12 (void)
{
  int k;
  void *pscr = NULL;
  allcanal = (canal_12 *) malloc (sizeof (canal_12) * nprc_);

  k = 1;
  double tot_width = calcwidth12 ();
  decay12information (tot_width);
  do
    {
      menu1 (56, 6, "", "\026"
	     " Model parameters     "
	     " Parameter dependence "
	     " Event generator      ", "n_13_*", &pscr, &k);
      switch (k)
	{
	case 1:		/*  ******* Change parameter ********  */
	  if (change_parameter (55, 10))
	    {
              double new_width = calcwidth12 ();
	      decay12information (new_width);
	    }
	  break;

	case 2:
	  paramdependence (totwidth, processch, "Width  [GeV]");
	  break;

	case 3:
	  menu_1to2_EventGenerator ();
	}
    }
  while (k != 0);
  free (allcanal);
  clrbox (1, 16, 80, 24);
}
